@@ -344,8 +344,10 @@ sub test_reconstruct_splitconv
     my ($self) = @_;
     my %exp;
 
+    my $talk = $self->{store}->get_client();
+
     # check IMAP server has the XCONVERSATIONS capability
-    $self->assert($self->{store}->get_client()->capability()->{xconversations});
+    $self->assert($talk->capability()->{xconversations});
 
     xlog $self, "generating message A";
     $exp{A} = $self->make_message("Message A");
@@ -358,12 +360,18 @@ sub test_reconstruct_splitconv
       $exp{"A$_"}->set_attributes(uid => 1+$_, cid => $exp{A}->make_cid());
     }
 
+    $talk->create('foo');
+    $talk->copy('1:*', 'foo');
+
     $self->check_messages(\%exp, keyed_on => 'uid');
 
     # first run WITHOUT splitting
     $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb', '-R', '-r');
 
     $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("foo");
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("INBOX");
 
     # then run WITH splitting, and see the changed CIDs
     $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb', '-R', '-r', '-S');
@@ -380,6 +388,38 @@ sub test_reconstruct_splitconv
     $exp{"A20"}->set_attributes(cid => $exp{"A20"}->make_cid(), basecid => $exp{A}->make_cid());
 
     $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("foo");
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("INBOX");
+
+    # zero everything out
+    $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb', '-z', 'cassandane');
+
+    # rebuild
+    $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb', '-b', 'cassandane');
+
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("foo");
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("INBOX");
+
+    # support for -Z was added after 3.8
+    my ($maj, $min) = Cassandane::Instance->get_version();
+    return if ($maj < 3 or ($maj == 3 and $min < 8));
+
+    # zero out ONLY two CIDs
+    $self->{instance}->run_command({ cyrus => 1 }, 'ctl_conversationsdb',
+                                    '-Z' => $exp{"A15"}->make_cid(),
+                                    '-Z' => $exp{"A10"}->make_cid(),
+                                    'cassandane');
+    for (10..19) {
+      $exp{"A$_"}->set_attributes(cid => undef, basecid => undef);
+    }
+
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("foo");
+    $self->check_messages(\%exp, keyed_on => 'uid');
+    $talk->select("INBOX");
 }
 
 #
@@ -866,11 +906,8 @@ sub test_guid_duplicate_same_folder
     $self->assert_null($r3);
     $self->assert_matches(qr/Too many identical emails/, $talk->get_last_error());
 
-    if ($self->{instance}->{have_syslog_replacement}) {
-        my @lines = $self->{instance}->getsyslog();
-        $self->assert_matches(qr{IOERROR: conversations GUID limit},
-                              join("\n", @lines));
-    }
+    $self->assert_syslog_matches($self->{instance},
+                                 qr{IOERROR: conversations GUID limit});
 
     $talk->select("INBOX.dest");
     my $data = $talk->fetch("1:*", "(emailid threadid uid)");
@@ -912,12 +949,8 @@ sub test_guid_duplicate_total_count
     $self->assert_not_null($r4);
     $self->assert_null($r5);
     $self->assert_matches(qr/Too many identical emails/, $talk->get_last_error());
-
-    if ($self->{instance}->{have_syslog_replacement}) {
-        my @lines = $self->{instance}->getsyslog();
-        $self->assert_matches(qr{IOERROR: conversations GUID limit},
-                              join("\n", @lines));
-    }
+    $self->assert_syslog_matches($self->{instance},
+                                 qr{IOERROR: conversations GUID limit});
 }
 
 #
@@ -952,11 +985,8 @@ sub test_guid_duplicate_expunges
     $self->assert_null($r);
     $self->assert_matches(qr/Too many identical emails/, $talk->get_last_error());
 
-    if ($self->{instance}->{have_syslog_replacement}) {
-        my @lines = $self->{instance}->getsyslog();
-        $self->assert_matches(qr{IOERROR: conversations GUID limit},
-                              join("\n", @lines));
-    }
+    $self->assert_syslog_matches($self->{instance},
+                                 qr{IOERROR: conversations GUID limit});
 }
 
 # Test APPEND of two messages, the second of which has a different subject,
