@@ -881,7 +881,7 @@ sub test_restore_calendars_all_dryrun
     $self->assert_null($imip);
 }
 
-sub test_restore_calendars_batch_size_bug
+sub test_restore_calendars_batch_size_bug1
     :min_version_3_7 :needs_component_jmap
 {
     my ($self) = @_;
@@ -901,6 +901,7 @@ sub test_restore_calendars_batch_size_bug
     my $calid = $res->[0][1]{created}{"1"}{id};
 
     xlog "create a bunch of events";
+    # two more than current Cyrus batch size (512)
     my %events = ();
     foreach my $n (1..514) {
         $events{"$n"} = {
@@ -932,6 +933,7 @@ sub test_restore_calendars_batch_size_bug
     $self->assert_not_null($id513);
 
     xlog "delete event 513";
+    # leaving first 512 events for batch 1 and #514 for batch 2
     $res = $jmap->CallMethods([
         ['CalendarEvent/set', {
             destroy => [$id513]
@@ -968,6 +970,95 @@ sub test_restore_calendars_batch_size_bug
     $self->assert_num_equals(513, $res->[0][1]{numDestroysUndone});
 }
 
+sub test_restore_calendars_batch_size_bug2
+    :min_version_3_7 :needs_component_jmap
+{
+    my ($self) = @_;
+
+    my $jmap = $self->{jmap};
+
+    xlog "create calendar";
+    my $res = $jmap->CallMethods([
+        ['Calendar/set', {
+            create => {
+                "1" => {
+                    name => "foo"
+                }
+            }
+         }, "R1"]
+    ]);
+    my $calid = $res->[0][1]{created}{"1"}{id};
+
+    xlog "create a bunch of events";
+    # one more than current Cyrus batch size (512)
+    my %events = ();
+    foreach my $n (1..513) {
+        $events{"$n"} = {
+            "calendarIds" => {
+                $calid => JSON::true,
+            },
+            "title" => "foo",
+            "start" => "2015-10-06T16:45:00",
+            "timeZone" => "Australia/Melbourne",
+            "duration" => "PT15M"
+        }
+    }
+
+    $res = $jmap->CallMethods([
+        ['CalendarEvent/set', {
+            create => {
+                %events
+            }}, "R1"]
+    ]);
+
+    sleep(1);
+
+    xlog "fetch the id of event 513";
+    $res = $jmap->CallMethods([
+        ['CalendarEvent/query', {
+        }, 'R1']
+    ]);
+    my $id513 = $res->[0][1]{ids}[512];
+    $self->assert_not_null($id513);
+
+    xlog "delete event 513";
+    # leaving first 512 events for batch 1 and none for batch 2
+    $res = $jmap->CallMethods([
+        ['CalendarEvent/set', {
+            destroy => [$id513]
+         }, 'R2']
+    ]);
+    $self->assert_str_equals($id513, $res->[0][1]{destroyed}[0]);
+
+    xlog $self, "expire 513 from disk";
+    $self->{instance}->run_command({ cyrus => 1 }, 'cyr_expire', '-X' => '0d' );
+
+    xlog "delete calendar";
+    $res = $jmap->CallMethods([
+        ['Calendar/set', {
+            destroy => ["$calid"],
+            onDestroyRemoveEvents => JSON::true,
+         }, "R2."]
+    ]);
+    $self->assert_str_equals($calid, $res->[0][1]{destroyed}[0]);
+
+    xlog "restore calendar";
+    $res = $jmap->CallMethods([
+        ['Backup/restoreCalendars', {
+            performDryRun => JSON::true,
+            undoPeriod => "PT3S",
+            undoAll => JSON::true
+         }, "R4"]
+    ]);
+
+    $self->assert_not_null($res);
+    $self->assert_str_equals('Backup/restoreCalendars', $res->[0][0]);
+    $self->assert_str_equals('R4', $res->[0][2]);
+    $self->assert_num_equals(0, $res->[0][1]{numCreatesUndone});
+    $self->assert_num_equals(0, $res->[0][1]{numUpdatesUndone});
+    $self->assert_num_equals(512, $res->[0][1]{numDestroysUndone});
+}
+
 sub test_restore_mail_simple
     :min_version_3_3 :needs_component_jmap
 {
@@ -1710,13 +1801,13 @@ sub test_restore_mail_full
          }, "R5.5"],
     ]);
     $self->assert_num_equals(4, scalar(@{$res->[0][1]{destroyed}}));
-    $self->assert_str_equals($emailId2, $res->[0][1]{destroyed}[0]);
-    $self->assert_str_equals($draftId1, $res->[0][1]{destroyed}[1]);
-    $self->assert_str_equals($draftId2, $res->[0][1]{destroyed}[2]);
-    $self->assert_str_equals($draftId3, $res->[0][1]{destroyed}[3]);
+    my @expect = sort ($emailId2, $draftId1, $draftId2, $draftId3);
+    my @got = sort @{$res->[0][1]{destroyed}};
+    $self->assert_deep_equals(\@expect, \@got);
+
     $self->assert_num_equals(2, scalar @{$res->[1][1]{destroyed}});
-    my @expect = sort ($fooId, $barId);
-    my @got = sort @{$res->[1][1]{destroyed}};
+    @expect = sort ($fooId, $barId);
+    @got = sort @{$res->[1][1]{destroyed}};
     $self->assert_deep_equals(\@expect, \@got);
 
     xlog "create a new 'bar' mailbox";
@@ -2317,7 +2408,7 @@ sub test_restore_mail_twice
     ]);
 
     # need a gap between move and destroy otherwise we will restore both copies
-    sleep 1;
+    sleep 6;
 
     xlog "destroy email1";
     $res = $jmap->CallMethods([
@@ -2329,7 +2420,7 @@ sub test_restore_mail_twice
     xlog "restore mail prior to most recent changes";
     $res = $jmap->CallMethods([
         ['Backup/restoreMail', {
-            undoPeriod => "PT1S"
+            undoPeriod => "PT3S"
          }, "R7"],
         ['Email/get', {
             ids => [$emailId1],
@@ -2367,7 +2458,7 @@ sub test_restore_mail_twice
     $self->assert_null($res->[1][1]{list}[0]{mailboxIds}->{$inbox->{id}});
 
     # need a gap between destroys otherwise we will restore both copies
-    sleep 1;
+    sleep 6;
 
     xlog "destroy email1 again";
     $res = $jmap->CallMethods([
@@ -2379,7 +2470,7 @@ sub test_restore_mail_twice
     xlog "restore mail prior to most recent changes";
     $res = $jmap->CallMethods([
         ['Backup/restoreMail', {
-            undoPeriod => "PT1S"
+            undoPeriod => "PT3S"
          }, "R7"],
         ['Email/get', {
             ids => [$emailId1],
