@@ -1321,12 +1321,85 @@ sub test_sighup_reloading_proto
         $self->lemming_census());
 }
 
+sub test_pidfile_symlink
+{
+    my ($self) = @_;
+
+    $self->{instance}->_init_basedir_and_name();
+
+    # XXX it would be nice to put the symlink in the basedir, but anything
+    # XXX we put in there before start will be blown away by _build_skeleton.
+    # XXX instead, set stuff up in /tmp where it won't get trampled before use
+    my $tmp = "/tmp/cassandane-$self->{instance}->{name}";
+    my $symlink_target = "$tmp/rogue_master_pidfile";
+    my $pidfile = "$tmp/master.pid";
+
+    mkdir $tmp or die "mkdir $tmp: $!";
+    symlink $symlink_target, $pidfile
+        or die "symlink $symlink_target $pidfile: $!";
+
+    $self->{instance}->{config}->set('master_pid_file', $pidfile);
+
+    eval {
+        $self->start();
+    };
+    my $e = $@;
+    $self->assert_matches(qr{exited with code 71}, $e->text());
+
+    $self->assert_num_equals(0, $self->{instance}->is_running());
+    $self->assert_syslog_matches($self->{instance},
+                                 qr{can't open pidfile: Too many levels});
+
+    unlink $pidfile;
+    unlink $symlink_target;
+    rmdir $tmp;
+}
+
+sub test_pidfile_lock_symlink
+{
+    my ($self) = @_;
+
+    $self->{instance}->_init_basedir_and_name();
+
+    # XXX it would be nice to put the symlink in the basedir, but anything
+    # XXX we put in there before start will be blown away by _build_skeleton.
+    # XXX instead, set stuff up in /tmp where it won't get trampled before use
+    my $tmp = "/tmp/cassandane-$self->{instance}->{name}";
+    my $symlink_target = "$tmp/rogue_master_pidfile_lock";
+    my $pidfile = "$tmp/master.pid";
+    my $pidfile_lock = "$pidfile.lock";
+
+    mkdir $tmp or die "mkdir $tmp: $!";
+    symlink $symlink_target, $pidfile_lock
+        or die "symlink $symlink_target $pidfile_lock: $!";
+
+    $self->{instance}->{config}->set('master_pid_file', $pidfile);
+
+    eval {
+        $self->start();
+    };
+    my $e = $@;
+    $self->assert_matches(qr{exited with code 71}, $e->text());
+
+    $self->assert_num_equals(0, $self->{instance}->is_running());
+    $self->assert_syslog_matches(
+        $self->{instance},
+        qr{can't open pidfile lock: \S+ \(Too many levels}
+    );
+
+    unlink $pidfile_lock;
+    unlink $symlink_target;
+    rmdir $tmp;
+}
+
 sub test_ready_file_new
 {
     my ($self) = @_;
 
+    # config isn't fully initialised until start() is called, so we can't
+    # just read these the sensible way. instead, predict the defaults.
     my $ready_file = $self->{instance}->get_basedir() . '/conf/master.ready';
-    my $pid_file = $self->{instance}->_pid_file();
+    my $pid_file = $self->{instance}->get_basedir() . '/run/master.pid';
 
     # pid file should not already exist
     my $pid_sb = stat($pid_file);
@@ -1356,16 +1429,14 @@ sub test_ready_file_exists
 {
     my ($self) = @_;
 
-    # force basedir to be computed
-    $self->{instance}->get_basedir();
+    # config isn't fully initialised until start() is called, so we can't
+    # just read these the sensible way. instead, predict the defaults.
+    my $pid_file = $self->{instance}->get_basedir() . '/run/master.pid';
 
     # cannot be under basedir because it'll be blown away at startup
     my $ready_file = "/tmp/cassandane-$$-master.ready";
     $self->{instance}->{config}->set('master_ready_file', $ready_file);
 
-    # must be after the get_basedir() call above
-    my $pid_file = $self->{instance}->_pid_file();
-
     system("touch", $ready_file) == 0 or die "touch $ready_file: $?";
     sleep 3;
 
