@@ -626,6 +626,11 @@ sub _pid_file
 
     $name ||= 'master';
 
+    if ($name eq 'master') {
+        my $pidfile = $self->{config}->get('master_pid_file');
+        return $self->{config}->substitute($pidfile) if $pidfile;
+    }
+
     return $self->{basedir} . "/run/$name.pid";
 }
 
@@ -919,8 +924,6 @@ sub _read_pid_file
     my $file = $self->_pid_file($name);
     my $pid;
 
-    return undef if ( ! -f $file );
-
     open PID,'<',$file
         or return undef;
     while(<PID>)
@@ -961,7 +964,6 @@ sub _start_master
         # The following is added automatically by _fork_command:
         # '-C', $self->_imapd_conf(),
         '-l', '255',
-        '-p', $self->_pid_file(),
         '-d',
         '-M', $self->_master_conf(),
     );
@@ -970,7 +972,6 @@ sub _start_master
         xlog "_start_master: logging to $logfile";
         push(@cmd, '-L', $logfile);
     }
-    unlink $self->_pid_file();
     # Start master daemon
     $self->run_command({ cyrus => 1 }, @cmd);
 
@@ -1097,7 +1098,7 @@ sub _start_smtpd
 
     my $basedir = $self->{basedir};
 
-    my $host = 'localhost';
+    my $host = '127.0.0.1';
 
     my $port = Cassandane::PortManager::alloc($host);
 
@@ -1160,7 +1161,7 @@ sub start_httpd {
 
     my $basedir = $self->{basedir};
 
-    my $host = 'localhost';
+    my $host = '127.0.0.1';
     $port ||= Cassandane::PortManager::alloc($host);
 
     my $httpdpid = fork();
