@@ -68,9 +68,6 @@
 #define DFLAG_UNBIND    "DAV:unbind"
 #define DFLAG_UNCHANGED "DAV:unchanged"
 
-/* XML namespace URIs */
-#define XML_NS_CYRUS    "http://cyrusimap.org/ns/"
-
 /* Supported TLS version for Upgrade */
 #define TLS_VERSION      "TLS/1.2"
 
@@ -280,7 +277,7 @@ struct range {
 
 struct patch_doc_t {
     const char *format;                 /* MIME format of patch document */
-    int (*proc)();                      /* Function to parse and apply doc */
+    int (*proc)(txn_t *, void *);       /* Function to parse and apply doc */
 };
 
 typedef int (*encode_proc_t)(struct transaction_t *txn,
@@ -578,7 +575,6 @@ extern struct auth_state *httpd_authstate;
 extern struct namespace httpd_namespace;
 extern const char *httpd_localip, *httpd_remoteip;
 extern unsigned long config_httpmodules;
-extern int config_httpprettytelemetry;
 extern strarray_t *httpd_log_headers;
 extern char *httpd_altsvc;
 
@@ -586,6 +582,8 @@ extern int ignorequota;
 extern int apns_enabled;
 extern int ws_enabled;
 
+extern struct proc_handle *httpd_proc_handle;
+
 extern xmlURIPtr parse_uri(unsigned meth, const char *uri, unsigned path_reqd,
                            const char **errstr);
 extern dynarray_t *parse_accept(const char **hdr);
