@@ -88,6 +88,7 @@
 #include "sync_support.h"
 #include "seen.h"
 #include "userdeny.h"
+#include "prometheus.h"
 
 /* generated headers are not necessarily in current directory */
 #include "imap/imap_err.h"
@@ -462,6 +463,8 @@ int service_init(int argc __attribute__((unused)),
         }
     }
 
+    prometheus_increment(CYRUS_POP3_READY_LISTENERS);
+
     return 0;
 }
 
@@ -476,6 +479,10 @@ int service_main(int argc __attribute__((unused)),
     sasl_security_properties_t *secprops=NULL;
     struct mboxevent *mboxevent = NULL;
 
+    /* fatal/shut_down will adjust these, so we need to set them early */
+    prometheus_decrement(CYRUS_POP3_READY_LISTENERS);
+    prometheus_increment(CYRUS_POP3_ACTIVE_CONNECTIONS);
+
     if (config_iolog) {
         io_count_start = xmalloc (sizeof (struct io_count));
         io_count_stop = xmalloc (sizeof (struct io_count));
@@ -493,6 +500,9 @@ int service_main(int argc __attribute__((unused)),
     count_top = 0;
     count_dele = 0;
 
+    /* count the connection, now that it's established */
+    prometheus_increment(CYRUS_POP3_CONNECTIONS_TOTAL);
+
     /* Find out name of client host */
     popd_clienthost = get_clienthost(0, &localip, &remoteip);
 
@@ -553,6 +563,8 @@ int service_main(int argc __attribute__((unused)),
 
     /* QUIT executed */
 
+    prometheus_decrement(CYRUS_POP3_ACTIVE_CONNECTIONS);
+
     /* send a Logout event notification */
     if ((mboxevent = mboxevent_new(EVENT_LOGOUT))) {
         mboxevent_set_access(mboxevent,
@@ -580,6 +592,8 @@ int service_main(int argc __attribute__((unused)),
         free(io_count_stop);
     }
 
+    prometheus_increment(CYRUS_POP3_READY_LISTENERS);
+
     return 0;
 }
 
@@ -591,6 +605,9 @@ void service_abort(int error)
 
 static void usage(void)
 {
+    if (kflag) prometheus_decrement(CYRUS_POP3_ACTIVE_CONNECTIONS);
+    prometheus_increment(CYRUS_POP3_SHUTDOWN_TOTAL_STATUS_ERROR);
+
     prot_printf(popd_out, "-ERR usage: pop3d [-C <alt_config>] [-k] [-s]\r\n");
     prot_flush(popd_out);
     exit(EX_USAGE);
@@ -646,6 +663,12 @@ void shut_down(int code)
         prot_flush(popd_out);
         bytes_out = prot_bytes_out(popd_out);
         prot_free(popd_out);
+        /* one less active connection */
+        prometheus_decrement(CYRUS_POP3_ACTIVE_CONNECTIONS);
+    }
+    else {
+        /* one less ready listener */
+        prometheus_decrement(CYRUS_POP3_READY_LISTENERS);
     }
 
     if (config_auditlog)
@@ -657,6 +680,9 @@ void shut_down(int code)
     tls_shutdown_serverengine();
 #endif
 
+    prometheus_increment(code ? CYRUS_POP3_SHUTDOWN_TOTAL_STATUS_ERROR
+                              : CYRUS_POP3_SHUTDOWN_TOTAL_STATUS_OK);
+
     saslprops_free(&saslprops);
 
     cyrus_done();
@@ -680,6 +706,16 @@ EXPORTED void fatal(const char* s, int code)
 
     if (recurse_code) {
         /* We were called recursively. Just give up */
+        if (popd_out) {
+            /* one less active connection */
+            prometheus_decrement(CYRUS_POP3_ACTIVE_CONNECTIONS);
+        }
+        else {
+            /* one less ready listener */
+            prometheus_decrement(CYRUS_POP3_READY_LISTENERS);
+        }
+        prometheus_increment(CYRUS_POP3_SHUTDOWN_TOTAL_STATUS_ERROR);
+
         proc_cleanup(&proc_handle);
         exit(recurse_code);
     }
@@ -1032,7 +1068,7 @@ done:
             else if (!strcmp(inputbuf, "auth")) {
                 cmd_auth(arg);
             }
-            else if (!strcmp(inputbuf, "stls") && tls_enabled()) {
+            else if (!strcmp(inputbuf, "stls") && tls_starttls_enabled()) {
                 if (arg) {
                     prot_printf(popd_out, "-ERR Unexpected extra argument\r\n");
                 } else {
@@ -1594,7 +1630,7 @@ static void cmd_capa(void)
         prot_write(popd_out, mechlist, strlen(mechlist));
     }
 
-    if (tls_enabled() && !popd_starttls_done && !popd_authstate) {
+    if (tls_starttls_enabled() && !popd_starttls_done && !popd_authstate) {
         prot_printf(popd_out, "STLS\r\n");
     }
     if (expire < 0) {
