@@ -275,7 +275,7 @@ int main(int argc, char **argv)
     global_sasl_init(1,0,NULL);
 
     /* Set namespace -- force standard (internal) */
-    if ((r = mboxname_init_namespace(&recon_namespace, 1)) != 0) {
+    if ((r = mboxname_init_namespace(&recon_namespace, NAMESPACE_OPTION_ADMIN))) {
         syslog(LOG_ERR, "%s", error_message(r));
         fatal(error_message(r), EX_CONFIG);
     }
@@ -821,7 +821,7 @@ static void reconstruct_mbentry(const char *header_path)
                         mbentry->mbtype &= ~(MBTYPE_LEGACY_DIRS);
 
                         token = tok_next(&tok);
-                        if (strcmpnull(token, FNAME_HEADER+1) || tok_next(&tok)) {
+                        if (strcmpnull(token, &FNAME_HEADER[1]) || tok_next(&tok)) {
                             xzfree(mbentry->uniqueid);
                         }
                     }
