@@ -99,6 +99,7 @@
 #include "version.h"
 #include "xmalloc.h"
 #include "xstrlcat.h"
+#include "xunlink.h"
 
 /* generated headers are not necessarily in current directory */
 #include "imap/imap_err.h"
@@ -135,6 +136,7 @@ static int sync_starttls_done = 0;
 static int sync_compress_done = 0;
 static int sync_sieve_mailbox_enabled = 0;
 static int sync_archive_enabled = 0;
+static struct proc_handle *proc_handle = NULL;
 
 static int opt_force = 0;
 
@@ -179,7 +181,7 @@ static struct sasl_callback mysasl_cb[] = {
 
 static void sync_reset(void)
 {
-    proc_cleanup();
+    proc_cleanup(&proc_handle);
 
     if (sync_in) {
         prot_NONBLOCK(sync_in);
@@ -241,7 +243,7 @@ int service_init(int argc __attribute__((unused)),
     int opt, r;
 
     if (geteuid() == 0) fatal("must run as the Cyrus user", EX_USAGE);
-    setproctitle_init(argc, argv, envp);
+    proc_settitle_init(argc, argv, envp);
 
     /* set signal handlers */
     signals_set_shutdown(&shut_down_via_signal);
@@ -264,7 +266,7 @@ int service_init(int argc __attribute__((unused)),
     }
 
     /* Set namespace -- force standard (internal) */
-    if ((r = mboxname_init_namespace(sync_namespacep, 1)) != 0) {
+    if ((r = mboxname_init_namespace(sync_namespacep, NAMESPACE_OPTION_ADMIN))) {
         fatal(error_message(r), EX_CONFIG);
     }
 
@@ -289,7 +291,7 @@ static void dobanner(void)
             prot_printf(sync_out, "%s", mechlist);
         }
 
-        if (tls_enabled() && !sync_starttls_done) {
+        if (tls_starttls_enabled() && !sync_starttls_done) {
             prot_printf(sync_out, "* STARTTLS\r\n");
         }
 
@@ -322,7 +324,7 @@ int service_main(int argc __attribute__((unused)),
 {
     const char *localip, *remoteip;
     sasl_security_properties_t *secprops = NULL;
-    int timeout;
+    int r, timeout;
 
     signals_poll();
 
@@ -363,7 +365,10 @@ int service_main(int argc __attribute__((unused)),
         tcp_disable_nagle(1); /* XXX magic fd */
     }
 
-    proc_register(config_ident, sync_clienthost, NULL, NULL, NULL);
+    r = proc_register(&proc_handle, 0,
+                      config_ident, sync_clienthost, NULL, NULL, NULL);
+    if (r) fatal("unable to register process", EX_IOERR);
+    proc_settitle(config_ident, sync_clienthost, NULL, NULL, NULL);
 
     /* Set inactivity timer */
     timeout = config_getduration(IMAPOPT_SYNC_TIMEOUT, 's');
@@ -410,7 +415,7 @@ void shut_down(int code)
 
     libcyrus_run_delayed();
 
-    proc_cleanup();
+    proc_cleanup(&proc_handle);
 
     seen_done();
 
@@ -453,7 +458,7 @@ EXPORTED void fatal(const char* s, int code)
 
     if (recurse_code) {
         /* We were called recursively. Just give up */
-        proc_cleanup();
+        proc_cleanup(&proc_handle);
         exit(recurse_code);
     }
     recurse_code = code;
@@ -649,7 +654,7 @@ static void cmdloop(void)
             break;
 
         case 'S':
-            if (!strcmp(cmd.s, "Starttls") && tls_enabled()) {
+            if (!strcmp(cmd.s, "Starttls") && tls_starttls_enabled()) {
                 if (c == '\r') c = prot_getc(sync_in);
                 if (c != '\n') goto extraargs;
 
@@ -781,7 +786,10 @@ static void cmd_authenticate(char *mech, char *resp)
     }
 
     sync_userid = xstrdup((const char *) val);
-    proc_register(config_ident, sync_clienthost, sync_userid, NULL, NULL);
+    r = proc_register(&proc_handle, 0,
+                      config_ident, sync_clienthost, sync_userid, NULL, NULL);
+    if (r) fatal("unable to register process", EX_IOERR);
+    proc_settitle(config_ident, sync_clienthost, sync_userid, NULL, NULL);
 
     syslog(LOG_NOTICE, "login: %s %s %s%s %s", sync_clienthost, sync_userid,
            mech, sync_starttls_done ? "+TLS" : "", "User logged in");
@@ -956,7 +964,7 @@ static void cmd_restart(struct sync_reserve_list **reserve_listp, int re_alloc)
         for (msg = res->list->head; msg; msg = msg->next) {
             if (!msg->fname) continue;
             pl = partition_list_add(res->part, pl);
-            unlink(msg->fname);
+            xunlink(msg->fname);
         }
     }
     sync_reserve_list_free(reserve_listp);
